@@ -1,6 +1,5 @@
 #include <R.h>
 #include "Graph.h"
-#include "dists.h"
 #include "Pp.h"
 
 
@@ -8,8 +7,6 @@ extern "C" {
 
 SEXP graph_c(SEXP Args)
 {
-//	e<-.Extern("graph_c", pp, typei, par , preprocessR, as.integer(toroidal), as.integer(doDists), dbg)
-
 	Pp pp;
 	double *prepR, *par;
 	int *gtype, *doDists, *toroidal, *dbg;
@@ -26,25 +23,24 @@ SEXP graph_c(SEXP Args)
 	par = REAL(CAR(Args)); // graph par
 
 	Args = CDR(Args);
-	prepR = REAL(CAR(Args)); // if preprocessing 
-	
+	prepR = REAL(CAR(Args)); // if preprocessing
+
 	Args = CDR(Args);
 	toroidal = INTEGER(CAR(Args)); // if toroidal correction
-	
+
 	Args = CDR(Args);
 	doDists = INTEGER(CAR(Args)); // if the distances are precalculated and stored
-	
+
 	Args = CDR(Args);
 	dbg = INTEGER(CAR(Args)); // if debug messages
-				
+
 //	void Init(Pp *pp0, double *par, double *prepR, int *doDists, int *toroidal, int *dbg );
 	graph.Init(&pp, gtype, par, prepR, doDists, toroidal, dbg);
 	graph.sg_calc();
-	
+
 	if(*dbg)printf("\n");
 	return graph.toSEXP();
 }
 
-	
 
 }
